@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2004, 2022, Oracle and/or its affiliates.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -59,15 +59,12 @@ X Window System is a trademark of The Open Group.
 #include "config.h"
 #endif
 
-#if defined(TCPCONN) || defined(STREAMSCONN)
+#ifdef TCPCONN
 #define NEEDSOCKETS
 #endif
 #ifdef UNIXCONN
 #define NEEDSOCKETS
 #endif
-#ifdef DNETCONN
-#define NEEDSOCKETS
-#endif
 
 #include <X11/Xlib.h>
 #include <X11/Xos.h>
@@ -76,45 +73,25 @@ X Window System is a trademark of The Open Group.
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
-#ifdef X_NOT_POSIX
-#include <setjmp.h>
-#endif
 #include <ctype.h>
 #include <X11/Xauth.h>
 #include <X11/Xmu/Error.h>
 #include <stdlib.h>
 
 #ifdef NEEDSOCKETS
-#ifdef att
-typedef unsigned short unsign16;
-typedef unsigned long unsign32;
-typedef short sign16;
-typedef long sign32;
-#include <interlan/socket.h>
-#include <interlan/netdb.h>
-#include <interlan/in.h>
+#ifdef WIN32
+#include <X11/Xwinsock.h>
 #else
-#ifndef Lynx
 #include <sys/socket.h>
-#else
-#include <socket.h>
-#endif
 #include <netdb.h>
 #include <netinet/in.h>
 #endif
 #endif /* NEEDSOCKETS */
 
-#ifndef BAD_ARPAINET
+#ifndef WIN32
 #include <arpa/inet.h>
-#else
-/* bogus definition of inet_makeaddr() in BSD 4.2 and Ultrix */
-extern unsigned long inet_makeaddr();
 #endif
 
-#ifdef DNETCONN
-#include <netdnet/dn.h>
-#include <netdnet/dnetdb.h>
-#endif
 
 #ifdef SECURE_RPC
 #include <pwd.h>
@@ -124,7 +101,7 @@ extern unsigned long inet_makeaddr();
 #include <limits.h>
 #undef _POSIX_C_SOURCE
 #else
-#if defined(X_NOT_POSIX) || defined(_POSIX_SOURCE)
+#ifdef _POSIX_SOURCE
 #include <limits.h>
 #else
 #define _POSIX_SOURCE
@@ -142,20 +119,21 @@ extern int getdomainname(char *name, size_t len);
 #endif
 #endif
 
+#if defined(IPv6) && !defined(AF_INET6)
+#error "Cannot build IPv6 support without AF_INET6"
+#endif
+
+#ifdef USE_GETTEXT
+# include <locale.h>	/* setlocale()  */
+# include <libintl.h>	/* gettext(), textdomain(), etc. */
+#else
+# define gettext(a) (a)
+#endif
+
 static int change_host(Display *dpy, char *name, Bool add);
 static const char *get_hostname(XHostAddress *ha);
 static int local_xerror(Display *dpy, XErrorEvent *rep);
-
-#ifdef RETSIGTYPE /* autoconf AC_TYPE_SIGNAL */
-# define signal_t RETSIGTYPE
-#else /* Imake */
-#ifdef SIGNALRETURNSINT
-#define signal_t int
-#else
-#define signal_t void
-#endif
-#endif /* RETSIGTYPE */
-static signal_t nameserver_lost(int sig);
+static void nameserver_lost(int sig);
 
 #define NAMESERVER_TIMEOUT 5	/* time to wait for nameserver */
 
@@ -164,10 +142,10 @@ static volatile int nameserver_timedout;
 static char *ProgramName;
 
 #ifdef NEEDSOCKETS
-static int 
+static int
 XFamily(int af)
 {
-    int i;
+    unsigned int i;
     static struct _familyMap {
 	int af, xf;
     } familyMap[] = {
@@ -179,7 +157,7 @@ XFamily(int af)
 #endif
 #ifdef	AF_INET
         { AF_INET, FamilyInternet },
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
         { AF_INET6, FamilyInternet6 },
 #endif
 #endif
@@ -193,7 +171,13 @@ XFamily(int af)
 }
 #endif /* NEEDSOCKETS */
 
-static Display *dpy;
+#if defined(__CYGWIN__) || defined(WIN32)
+void sethostent(int x)
+{}
+
+void endhostent()
+{}
+#endif
 
 int
 main(int argc, char *argv[])
@@ -204,39 +188,45 @@ main(int argc, char *argv[])
     int nfailed = 0;
     XHostAddress *list;
     Bool enabled = False;
-#ifdef DNETCONN
-    char *dnet_htoa();
-    struct nodeent *np;
-    struct dn_naddr *nlist, dnaddr, *dnaddrp, *dnet_addr();
-    char *cp;
+    Display *dpy;
+
+#ifdef USE_GETTEXT
+    const char *domaindir;
+
+    setlocale(LC_ALL, "");
+    textdomain("xhost");
+
+    /* mainly for debugging */
+    if ((domaindir = getenv("TEXTDOMAINDIR")) == NULL) {
+	domaindir = LOCALEDIR;
+    }
+    bindtextdomain("xhost", domaindir);
 #endif
- 
+
     ProgramName = argv[0];
 
     if (argc == 2 && !strcmp(argv[1], "-help")) {
-	fprintf(stderr, "usage: %s [[+-]hostname ...]\n", argv[0]);
+	fprintf(stderr, gettext("usage: %s [[+-]hostname ...]\n"), argv[0]);
 	exit(1);
     }
 
     if ((dpy = XOpenDisplay(NULL)) == NULL) {
-	fprintf(stderr, "%s:  unable to open display \"%s\"\n",
+	fprintf(stderr,
+		gettext("%s:  unable to open display \"%s\"\n"),
 		ProgramName, XDisplayName (NULL));
 	exit(1);
     }
 
     XSetErrorHandler(local_xerror);
- 
- 
+
+
     if (argc == 1) {
-#ifdef DNETCONN
-	setnodeent(1);		/* keep the database accessed */
-#endif
 	sethostent(1);		/* don't close the data base each time */
 	list = XListHosts(dpy, &nhosts, &enabled);
 	if (enabled)
-	    printf ("access control enabled, only authorized clients can connect\n");
+	    printf (gettext("access control enabled, only authorized clients can connect\n"));
 	else
-	    printf ("access control disabled, clients can connect from any host\n");
+	    printf (gettext("access control disabled, clients can connect from any host\n"));
 
 	if (nhosts != 0) {
 	    for (i = 0; i < nhosts; i++ )  {
@@ -265,16 +255,16 @@ main(int argc, char *argv[])
 			printf("SI:");
 			break;
 		    default:
-			printf("<unknown family type %d>:", list[i].family);
+			printf(gettext("<unknown family type %d>:"), list[i].family);
 			break;
 		    }
 		    printf ("%s", hostname);
 		} else {
-		    printf ("<unknown address in family %d>",
+		    printf (gettext("<unknown address in family %d>"),
 			    list[i].family);
 		}
 		if (nameserver_timedout) {
-		    printf("\t(no nameserver response within %d seconds)\n",
+		    printf(gettext("\t(no nameserver response within %d seconds)\n"),
 			   NAMESERVER_TIMEOUT);
 		    nameserver_timedout = 0;
 		} else
@@ -285,32 +275,32 @@ main(int argc, char *argv[])
 	}
 	exit(0);
     }
- 
+
     for (i = 1; i < argc; i++) {
 	arg = argv[i];
 	if (*arg == '-') {
-	    
+
 	    if (!argv[i][1] && ((i+1) == argc)) {
-		printf ("access control enabled, only authorized clients can connect\n");
+		printf (gettext("access control enabled, only authorized clients can connect\n"));
 		XEnableAccessControl(dpy);
 	    } else {
 		arg = argv[i][1]? &argv[i][1] : argv[++i];
 		if (!change_host (dpy, arg, False)) {
-		    fprintf (stderr, "%s:  bad hostname \"%s\"\n",
+		    fprintf (stderr, gettext("%s:  bad hostname \"%s\"\n"),
 			     ProgramName, arg);
 		    nfailed++;
 		}
 	    }
 	} else {
 	    if (*arg == '+' && !argv[i][1] && ((i+1) == argc)) {
-		printf ("access control disabled, clients can connect from any host\n");
+		printf (gettext("access control disabled, clients can connect from any host\n"));
 		XDisableAccessControl(dpy);
 	    } else {
 		if (*arg == '+') {
 		    arg = argv[i][1]? &argv[i][1] : argv[++i];
 		}
 		if (!change_host (dpy, arg, True)) {
-		    fprintf (stderr, "%s:  bad hostname \"%s\"\n",
+		    fprintf (stderr, gettext("%s:  bad hostname \"%s\"\n"),
 			     ProgramName, arg);
 		    nfailed++;
 		}
@@ -321,7 +311,7 @@ main(int argc, char *argv[])
     exit(nfailed);
 }
 
- 
+
 
 /*
  * change_host - edit the list of hosts that may connect to the server;
@@ -331,36 +321,27 @@ main(int argc, char *argv[])
  * address.
  */
 
-static int 
+static int
 change_host(Display *dpy, char *name, Bool add)
 {
     XHostAddress ha;
     char *lname;
-    int namelen, i, family = FamilyWild;
-#ifdef K5AUTH
-    krb5_principal princ;
-    krb5_data kbuf;
-#endif
+    size_t namelen, i;
+    int family = FamilyWild;
 #ifdef NEEDSOCKETS
     static struct in_addr addr;	/* so we can point at it */
-#if defined(IPv6) && defined(AF_INET6)
+#ifdef IPv6
     static struct in6_addr addr6; /* so we can point at it */
-#else
-    struct hostent *hp;
 #endif
 #endif
     char *cp;
-#ifdef DNETCONN
-    struct dn_naddr *dnaddrp;
-    struct nodeent *np;
-    static struct dn_naddr dnaddr;
-#endif				/* DNETCONN */
-    static const char *add_msg = "being added to access control list";
-    static const char *remove_msg = "being removed from access control list";
+    const char *add_msg = gettext("being added to access control list");
+    const char *remove_msg = gettext("being removed from access control list");
 
     namelen = strlen(name);
     if ((lname = (char *)malloc(namelen+1)) == NULL) {
-	fprintf (stderr, "%s: malloc bombed in change_host\n", ProgramName);
+	fprintf (stderr, gettext("%s: malloc bombed in change_host\n"),
+		 ProgramName);
 	exit (1);
     }
     for (i = 0; i < namelen; i++) {
@@ -368,22 +349,21 @@ change_host(Display *dpy, char *name, Bool add)
     }
     lname[namelen] = '\0';
     if (!strncmp("inet:", lname, 5)) {
-#if defined(TCPCONN) || defined(STREAMSCONN)
+#ifdef TCPCONN
 	family = FamilyInternet;
 	name += 5;
 #else
-	fprintf (stderr, "%s: not compiled for TCP/IP\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for TCP/IP\n"), ProgramName);
 	free(lname);
 	return 0;
 #endif
     }
     else if (!strncmp("inet6:", lname, 6)) {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) && \
-    defined(IPv6) && defined(AF_INET6)
+#if defined(TCPCONN) && defined(IPv6)
 	family = FamilyInternet6;
 	name += 6;
 #else
-	fprintf (stderr, "%s: not compiled for IPv6\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for IPv6\n"), ProgramName);
 	free(lname);
 	return 0;
 #endif
@@ -391,33 +371,27 @@ change_host(Display *dpy, char *name, Bool add)
 #ifdef ACCEPT_INETV6 /* Allow inetv6 as an alias for inet6 for compatibility
 			with original X11 over IPv6 draft. */
     else if (!strncmp("inetv6:", lname, 7)) {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) && \
-    defined(IPv6) && defined(AF_INET6)
+#if defined(TCPCONN) && defined(IPv6)
 	family = FamilyInternet6;
 	name += 7;
 #else
-	fprintf (stderr, "%s: not compiled for IPv6\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for IPv6\n"), ProgramName);
 	free(lname);
 	return 0;
 #endif
     }
 #endif /* ACCEPT_INETV6 */
     else if (!strncmp("dnet:", lname, 5)) {
-#ifdef DNETCONN
-	family = FamilyDECnet;
-	name += 5;
-#else
-	fprintf (stderr, "%s: not compiled for DECnet\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for DECnet\n"), ProgramName);
 	free(lname);
 	return 0;
-#endif
     }
     else if (!strncmp("nis:", lname, 4)) {
 #ifdef SECURE_RPC
 	family = FamilyNetname;
 	name += 4;
 #else
-	fprintf (stderr, "%s: not compiled for Secure RPC\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for Secure RPC\n"), ProgramName);
 	free(lname);
 	return 0;
 #endif
@@ -427,7 +401,7 @@ change_host(Display *dpy, char *name, Bool add)
 	family = FamilyKrb5Principal;
 	name +=4;
 #else
-	fprintf (stderr, "%s: not compiled for Kerberos 5\n", ProgramName);
+	fprintf (stderr, gettext("%s: not compiled for Kerberos 5\n"), ProgramName);
 	free(lname);
 	return 0;
 #endif
@@ -440,11 +414,21 @@ change_host(Display *dpy, char *name, Bool add)
 	name += 3;
     }
     if (family == FamilyWild && (cp = strchr(lname, ':'))) {
-	*cp = '\0';
-	fprintf (stderr, "%s: unknown address family \"%s\"\n",
-		 ProgramName, lname);
-	free(lname);
-	return 0;
+#ifdef IPv6
+	/*
+	 * Check to see if inet_pton() can grok it as an IPv6 address
+	 */
+	if (inet_pton(AF_INET6, lname, &addr6.s6_addr) == 1) {
+	    family = FamilyInternet6;
+	} else
+#endif
+	{
+	    *cp = '\0';
+	    fprintf (stderr, gettext("%s: unknown address family \"%s\"\n"),
+		     ProgramName, lname);
+	    free(lname);
+	    return 0;
+	}
     }
     free(lname);
 
@@ -454,8 +438,8 @@ change_host(Display *dpy, char *name, Bool add)
 
 	cp = strchr(name, ':');
 	if (cp == NULL || cp == name) {
-	    fprintf(stderr, 
-	   "%s: type must be specified for server interpreted family \"%s\"\n",
+	    fprintf(stderr, gettext(
+	  "%s: type must be specified for server interpreted family \"%s\"\n"),
 	      ProgramName, name);
 	    return 0;
 	}
@@ -476,42 +460,16 @@ change_host(Display *dpy, char *name, Bool add)
 	return 1;
     }
 
-#ifdef DNETCONN
-    if (family == FamilyDECnet || ((family == FamilyWild) &&
-	(cp = strchr(name, ':')) && (*(cp + 1) == ':') &&
-	!(*cp = '\0'))) {
-	ha.family = FamilyDECnet;
-	if (dnaddrp = dnet_addr(name)) {
-	    dnaddr = *dnaddrp;
-	} else {
-	    if ((np = getnodebyname (name)) == NULL) {
-		fprintf (stderr, "%s:  unable to get node name for \"%s::\"\n",
-			 ProgramName, name);
-		return 0;
-	    }
-	    dnaddr.a_len = np->n_length;
-	    memmove( dnaddr.a_addr, np->n_addr, np->n_length);
-	}
-	ha.length = sizeof(struct dn_naddr);
-	ha.address = (char *)&dnaddr;
-	if (add) {
-	    XAddHost (dpy, &ha);
-	    printf ("%s:: %s\n", name, add_msg);
-	} else {
-	    XRemoveHost (dpy, &ha);
-	    printf ("%s:: %s\n", name, remove_msg);
-	}
-	return 1;
-    }
-#endif				/* DNETCONN */
 #ifdef K5AUTH
     if (family == FamilyKrb5Principal) {
+	krb5_principal princ;
+	krb5_data kbuf;
 	krb5_error_code retval;
 
 	retval = krb5_parse_name(name, &princ);
 	if (retval) {
 	    krb5_init_ets();	/* init krb errs for error_message() */
-	    fprintf(stderr, "%s: cannot parse Kerberos name: %s\n",
+	    fprintf(stderr, gettext("%s: cannot parse Kerberos name: %s\n"),
 		    ProgramName, error_message(retval));
 	    return 0;
 	}
@@ -530,14 +488,15 @@ change_host(Display *dpy, char *name, Bool add)
     }
 #endif
     if (family == FamilyLocalHost) {
+	char empty[] = "";
 	ha.length = 0;
-	ha.address = "";
+	ha.address = empty;
 	ha.family = family;
 	if (add)
 	    XAddHost(dpy, &ha);
 	else
 	    XRemoveHost(dpy, &ha);
-	printf( "non-network local connections %s\n", add ? add_msg : remove_msg);
+	printf( gettext("non-network local connections %s\n"), add ? add_msg : remove_msg);
 	return 1;
     }
     /*
@@ -547,7 +506,7 @@ change_host(Display *dpy, char *name, Bool add)
 	(cp = strchr(name, '@'))) {
         char *netname = name;
 #ifdef SECURE_RPC
-	static char username[MAXNETNAMELEN];
+	static char username[MAXNETNAMELEN + 1];
 
 	if (!cp[1]) {
 	    struct passwd *pwd;
@@ -556,12 +515,12 @@ change_host(Display *dpy, char *name, Bool add)
 	    *cp = '\0';
 	    pwd = getpwnam(name);
 	    if (!pwd) {
-		fprintf(stderr, "no such user \"%s\"\n", name);
+		fprintf(stderr, gettext("no such user \"%s\"\n"), name);
 		return 0;
 	    }
 	    getdomainname(domainname, sizeof(domainname));
 	    if (!user2netname(username, pwd->pw_uid, domainname)) {
-		fprintf(stderr, "failed to get netname for \"%s\"\n", name);
+		fprintf(stderr, gettext("failed to get netname for \"%s\"\n"), name);
 		return 0;
 	    }
 	    netname = username;
@@ -582,13 +541,13 @@ change_host(Display *dpy, char *name, Bool add)
     }
 #ifdef NEEDSOCKETS
     /*
-     * First see if inet_addr() can grok the name; if so, then use it.
+     * First see if inet_aton/inet_addr can grok the name; if so, then use it.
      */
     if (((family == FamilyWild) || (family == FamilyInternet)) &&
-	((addr.s_addr = inet_addr(name)) != -1)) {
+	(inet_pton (AF_INET, name, &addr) == 1)) {
 	ha.family = FamilyInternet;
-	ha.length = 4;		/* but for Cray would be sizeof(addr.s_addr) */
-	ha.address = (char *)&addr; /* but for Cray would be &addr.s_addr */
+	ha.length = sizeof(addr.s_addr);
+	ha.address = (char *) &addr.s_addr;
 	if (add) {
 	    XAddHost (dpy, &ha);
 	    printf ("%s %s\n", name, add_msg);
@@ -597,16 +556,16 @@ change_host(Display *dpy, char *name, Bool add)
 	    printf ("%s %s\n", name, remove_msg);
 	}
 	return 1;
-    } 
-#if defined(IPv6) && defined(AF_INET6)
+    }
+#ifdef IPv6
     /*
      * Check to see if inet_pton() can grok it as an IPv6 address
      */
     else if (((family == FamilyWild) || (family == FamilyInternet6)) &&
 	     (inet_pton(AF_INET6, name, &addr6.s6_addr) == 1)) {
 	ha.family = FamilyInternet6;
-	ha.length = sizeof(addr6.s6_addr);		
-	ha.address = (char *) &addr6.s6_addr; 
+	ha.length = sizeof(addr6.s6_addr);
+	ha.address = (char *) &addr6.s6_addr;
 	if (add) {
 	    XAddHost (dpy, &ha);
 	    printf ("%s %s\n", name, add_msg);
@@ -615,39 +574,60 @@ change_host(Display *dpy, char *name, Bool add)
 	    printf ("%s %s\n", name, remove_msg);
 	}
 	return 1;
-    } else {
+    }
+#endif
+#ifdef HAVE_GETADDRINFO
+    else {
     /*
-     * Is it in the namespace?  
+     * Is it in the namespace?
      *
      * If no family was specified, use both Internet v4 & v6 addresses.
      * Otherwise, use only addresses matching specified family.
      */
 	struct addrinfo *addresses;
 	struct addrinfo *a;
+	struct addrinfo hints = {
+#  ifdef IPv6
+	    .ai_family = AF_UNSPEC
+#  else
+	    .ai_family = AF_INET
+#  endif
+	};
+
 	Bool didit = False;
 
-	if (getaddrinfo(name, NULL, NULL, &addresses) != 0)
+	if (getaddrinfo(name, NULL, &hints, &addresses) != 0)
 	    return 0;
 
 	for (a = addresses; a != NULL; a = a->ai_next) {
 	    if ( ((a->ai_family == AF_INET) && (family != FamilyInternet6))
-	      || ((a->ai_family == AF_INET6) && (family != FamilyInternet)) ) {
+#ifdef IPv6
+	      || ((a->ai_family == AF_INET6) && (family != FamilyInternet))
+#endif
+                ) {
+#ifdef DEBUG
 		char ad[INET6_ADDRSTRLEN];
+#endif
 		ha.family = XFamily(a->ai_family);
+#ifdef IPv6
 		if (a->ai_family == AF_INET6) {
 		    ha.address = (char *)
 		      &((struct sockaddr_in6 *) a->ai_addr)->sin6_addr;
-		    ha.length = 
+		    ha.length =
 		      sizeof (((struct sockaddr_in6 *) a->ai_addr)->sin6_addr);
-		} else {
+		} else
+#endif
+		{
 		    ha.address = (char *)
 		      &((struct sockaddr_in *) a->ai_addr)->sin_addr;
-		    ha.length = 
+		    ha.length =
 		      sizeof (((struct sockaddr_in *) a->ai_addr)->sin_addr);
 		}
+#ifdef DEBUG
 		inet_ntop(a->ai_family, ha.address, ad, sizeof(ad));
-	/* printf("Family: %d\nLength: %d\n", a->ai_family, ha.length); */
-		/* printf("Address: %s\n", ad); */
+                printf("Family: %d\nLength: %d\n", a->ai_family, ha.length);
+		printf("Address: %s\n", ad);
+#endif
 
 		if (add) {
 		    XAddHost (dpy, &ha);
@@ -668,20 +648,23 @@ change_host(Display *dpy, char *name, Bool add)
 		familyMsg = "inet ";
 	    }
 
-	    fprintf(stderr, "%s: unable to get %saddress for \"%s\"\n",
+	    fprintf(stderr, gettext("%s: unable to get %saddress for \"%s\"\n"),
 		ProgramName, familyMsg, name);
 	}
 	freeaddrinfo(addresses);
 	return 1;
     }
-#else /* !IPv6 */
+#else /* !HAVE_GETADDRINFO */
     /*
      * Is it in the namespace?
      */
-    else if (((hp = gethostbyname(name)) == (struct hostent *)NULL)
+    else {
+	struct hostent *hp = gethostbyname(name);
+
+	if ((hp == (struct hostent *)NULL)
 	     || hp->h_addrtype != AF_INET) {
-	return 0;
-    } else {
+	    return 0;
+	}
 	ha.family = XFamily(hp->h_addrtype);
 	ha.length = hp->h_length;
 #ifdef h_addr			/* new 4.3bsd version of gethostent */
@@ -709,7 +692,7 @@ change_host(Display *dpy, char *name, Bool add)
 	printf ("%s %s\n", name, add ? add_msg : remove_msg);
 	return 1;
     }
-#endif /* IPv6 */
+#endif /* HAVE_GETADDRINFO */
 #else /* NEEDSOCKETS */
     return 0;
 #endif /* NEEDSOCKETS */
@@ -722,37 +705,24 @@ change_host(Display *dpy, char *name, Bool add)
  * be found.
  */
 
-#ifdef X_NOT_POSIX
-jmp_buf env;
-#endif
 
 static const char *
 get_hostname(XHostAddress *ha)
 {
-#if (defined(TCPCONN) || defined(STREAMSCONN)) &&	\
-     (!defined(IPv6) || !defined(AF_INET6))
-    static struct hostent *hp = NULL;
-#endif
-#ifdef DNETCONN
-    struct nodeent *np;
-    static char nodeaddr[5 + 2 * DN_MAXADDL];
-#endif				/* DNETCONN */
-#ifdef K5AUTH
-    krb5_principal princ;
-    krb5_data kbuf;
-    char *kname;
-    static char kname_out[255];
-#endif
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
     struct sigaction sa;
 #endif
 
-#if defined(TCPCONN) || defined(STREAMSCONN)
-#if defined(IPv6) && defined(AF_INET6)
-    if ((ha->family == FamilyInternet) || (ha->family == FamilyInternet6)) {
+#ifdef TCPCONN
+#ifdef HAVE_GETNAMEINFO
+    if ((ha->family == FamilyInternet)
+#ifdef IPv6
+        || (ha->family == FamilyInternet6)
+#endif
+        ) {
 	struct sockaddr_storage saddr;
 	static char inetname[NI_MAXHOST];
-	int saddrlen;
+	unsigned int saddrlen;
 
 	inetname[0] = '\0';
 	memset(&saddr, 0, sizeof saddr);
@@ -763,78 +733,70 @@ get_hostname(XHostAddress *ha)
 #endif
 	    sin->sin_family = AF_INET;
 	    sin->sin_port = 0;
+	    if (sizeof(sin->sin_addr) > ha->length)
+		return "";
 	    memcpy(&sin->sin_addr, ha->address, sizeof(sin->sin_addr));
 	    saddrlen = sizeof(struct sockaddr_in);
-	} else {
+	}
+#ifdef IPv6
+	else {
 	    struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) &saddr;
 #ifdef SIN6_LEN
 	    sin6->sin6_len = sizeof(struct sockaddr_in6);
 #endif
 	    sin6->sin6_family = AF_INET6;
 	    sin6->sin6_port = 0;
+	    if (sizeof(sin6->sin6_addr) > ha->length)
+		return "";
 	    memcpy(&sin6->sin6_addr, ha->address, sizeof(sin6->sin6_addr));
 	    saddrlen = sizeof(struct sockaddr_in6);
 	}
+#endif
 
 	/* gethostbyaddr can take a LONG time if the host does not exist.
 	   Assume that if it does not respond in NAMESERVER_TIMEOUT seconds
 	   that something is wrong and do not make the user wait.
 	   gethostbyaddr will continue after a signal, so we have to
-	   jump out of it. 
+	   jump out of it.
 	   */
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
 	memset(&sa, 0, sizeof sa);
 	sa.sa_handler = nameserver_lost;
 	sa.sa_flags = 0;	/* don't restart syscalls */
 	sigaction(SIGALRM, &sa, NULL);
-#else
-	signal(SIGALRM, nameserver_lost);
-#endif
 	alarm(NAMESERVER_TIMEOUT);
-#ifdef X_NOT_POSIX
-	if (setjmp(env) == 0) 
 #endif
-	{ 
-	    getnameinfo((struct sockaddr *) &saddr, saddrlen, inetname,
-	      sizeof(inetname), NULL, 0, 0);
-	}
+	getnameinfo((struct sockaddr *) &saddr, saddrlen, inetname,
+		    sizeof(inetname), NULL, 0, 0);
+#ifdef SIGALRM
 	alarm(0);
+#endif
 	if (nameserver_timedout || inetname[0] == '\0')
 	    inet_ntop(((struct sockaddr *)&saddr)->sa_family, ha->address,
 		inetname, sizeof(inetname));
-	return inetname;	      
+	return inetname;
     }
 #else
     if (ha->family == FamilyInternet) {
-#ifdef CRAY
-	struct in_addr t_addr;
-	bzero((char *)&t_addr, sizeof(t_addr));
-	bcopy(ha->address, (char *)&t_addr, 4);
-	ha->address = (char *)&t_addr;
-#endif
+	static struct hostent *hp = NULL;
+
 	/* gethostbyaddr can take a LONG time if the host does not exist.
 	   Assume that if it does not respond in NAMESERVER_TIMEOUT seconds
 	   that something is wrong and do not make the user wait.
 	   gethostbyaddr will continue after a signal, so we have to
-	   jump out of it. 
+	   jump out of it.
 	   */
-#ifndef X_NOT_POSIX
+#ifdef SIGALRM
 	memset(&sa, 0, sizeof sa);
 	sa.sa_handler = nameserver_lost;
 	sa.sa_flags = 0;	/* don't restart syscalls */
 	sigaction(SIGALRM, &sa, NULL);
-#else
-	signal(SIGALRM, nameserver_lost);
-#endif
 	alarm(4);
-#ifdef X_NOT_POSIX
-	if (setjmp(env) == 0) {
-#endif
-	    hp = gethostbyaddr (ha->address, ha->length, AF_INET);
-#ifdef X_NOT_POSIX
-	}
 #endif
+	hp = gethostbyaddr (ha->address, ha->length, AF_INET);
+#ifdef SIGALRM
 	alarm(0);
+#endif
 	if (hp)
 	    return (hp->h_name);
 	else return (inet_ntoa(*((struct in_addr *)(ha->address))));
@@ -862,26 +824,19 @@ get_hostname(XHostAddress *ha)
 
 	    pwd = getpwuid(uid);
 	    if (pwd)
-		sprintf(netname, "%s@ (%*.*s)", pwd->pw_name,
-			ha->length, ha->length, ha->address);
+		snprintf(netname, sizeof(netname), "%s@ (%*.*s)",
+			 pwd->pw_name, ha->length, ha->length, ha->address);
 	}
 #endif
 	return (netname);
     }
-#ifdef DNETCONN
-    if (ha->family == FamilyDECnet) {
-	struct dn_naddr *addr_ptr = (struct dn_naddr *) ha->address;
-
-	if (np = getnodebyaddr(addr_ptr->a_addr, addr_ptr->a_len, AF_DECnet)) {
-	    sprintf(nodeaddr, "%s", np->n_name);
-	} else {
-	    sprintf(nodeaddr, "%s", dnet_htoa(ha->address));
-	}
-	return(nodeaddr);
-    }
-#endif
 #ifdef K5AUTH
     if (ha->family == FamilyKrb5Principal) {
+	krb5_principal princ;
+	krb5_data kbuf;
+	char *kname;
+	static char kname_out[255];
+
 	kbuf.data = ha->address;
 	kbuf.length = ha->length;
 	XauKrb5Decode(kbuf, &princ);
@@ -927,15 +882,10 @@ get_hostname(XHostAddress *ha)
 }
 
 /*ARGUSED*/
-static signal_t 
-nameserver_lost(int sig)
+static void
+nameserver_lost(_X_UNUSED int sig)
 {
     nameserver_timedout = 1;
-#ifdef X_NOT_POSIX
-    /* not needed with POSIX signals - stuck syscalls will not 
-       be restarted after signal delivery */
-    longjmp(env, -1);
-#endif
 }
 
 /*
@@ -943,21 +893,21 @@ nameserver_lost(int sig)
  * that an X_GetHosts request for an unknown address format was received, just
  * return, otherwise print the normal error message and continue.
  */
-static int 
+static int
 local_xerror(Display *dpy, XErrorEvent *rep)
 {
     if ((rep->error_code == BadAccess) && (rep->request_code == X_ChangeHosts)) {
-	fprintf (stderr, 
-		 "%s:  must be on local machine to add or remove hosts.\n",
+	fprintf (stderr,
+		 gettext("%s:  must be on local machine to add or remove hosts.\n"),
 		 ProgramName);
 	return 1;
-    } else if ((rep->error_code == BadAccess) && 
+    } else if ((rep->error_code == BadAccess) &&
 	       (rep->request_code == X_SetAccessControl)) {
-	fprintf (stderr, 
-	"%s:  must be on local machine to enable or disable access control.\n",
+	fprintf (stderr,
+		 gettext("%s:  must be on local machine to enable or disable access control.\n"),
 		 ProgramName);
 	return 1;
-    } else if ((rep->error_code == BadValue) && 
+    } else if ((rep->error_code == BadValue) &&
 	       (rep->request_code == X_ListHosts)) {
 	return 1;
     }
@@ -965,12 +915,3 @@ local_xerror(Display *dpy, XErrorEvent *rep)
     XmuPrintDefaultErrorMessage (dpy, rep, stderr);
     return 0;
 }
-
-#ifdef __CYGWIN__
-void sethostent(int x)
-{}
-
-void endhostent()
-{}
-#endif
-
